@@ -33,7 +33,7 @@ class BadResultException(Exception):
     pass
 
 class Query(object):
-    '''A query object has all of the methods necessary to programmatically 
+    ''' A query object has all of the methods necessary to programmatically 
         generate a mongo query as well as methods to retrieve results of the 
         query or do an update based on it.
         
@@ -41,13 +41,14 @@ class Query(object):
         not directly.
     '''
     def __init__(self, type, session):
-        '''**Parameters**:
+        ''' 
+            **Parameters**:
                 * type: A subclass of class:`mongoalchemy.document.Document`
                 * db: The :class:`~mongoalchemy.session.Session` which this query is associated with.
         '''
         self.session = session
         self.type = type
-        self.query = {}
+        self.__query = {}
         self.sort = []
         self._fields = None
         self.hints = []
@@ -57,6 +58,31 @@ class Query(object):
     def __iter__(self):
         return self.__get_query_result()
     
+    def resolve_name(self, name):
+        if not isinstance(name, basestring) or name[0] == '$':
+            return name
+        ret = self.type
+        for part in name.split('.'):
+            ret = getattr(ret, part)
+        return ret
+    
+    @property
+    def query(self):
+        def flatten(obj):
+            if not isinstance(obj, dict):
+                return obj
+            ret = {}
+            for k, v in obj.iteritems():
+                if not isinstance(k, basestring):
+                    k = str(k)
+                if isinstance(v, dict):
+                    v = flatten(v)
+                if isinstance(v, list):
+                    v = [flatten(x) for x in v]
+                ret[k] = v
+            return ret
+        return flatten(self.__query)
+    
     def __get_query_result(self):
         return self.session.execute_query(self)
     
@@ -70,17 +96,19 @@ class Query(object):
         return self._skip
     
     def limit(self, limit):
-        '''Sets the limit on the number of documents returned
-        **Parameters**:
-            * limit: the number of documents to return
+        ''' Sets the limit on the number of documents returned
+            
+            **Parameters**:
+                * limit: the number of documents to return
         '''
         self._limit = limit
         return self
     
     def skip(self, skip):
-        '''Sets the number of documents to skip in the result
-        **Parameters**:
-            * skip: the number of documents to skip
+        ''' Sets the number of documents to skip in the result
+        
+            **Parameters**:
+                * skip: the number of documents to skip
         '''
         self._skip = skip
         return self
@@ -91,7 +119,7 @@ class Query(object):
             affect each other
         '''
         qclone = Query(self.type, self.session)
-        qclone.query = deepcopy(self.query)
+        qclone.__query = deepcopy(self.__query)
         qclone.sort = deepcopy(self.sort)
         qclone._fields = deepcopy(self._fields)
         qclone._hints = deepcopy(self.hints)
@@ -116,7 +144,7 @@ class Query(object):
             there are multiple documents it simply returns the first one.  If
             there are no documents, first returns ``None``
         '''
-        for doc in self:
+        for doc in iter(self):
             return doc
         return None
     
@@ -124,22 +152,25 @@ class Query(object):
         return self.__get_query_result().__getitem__(index)
     
     def hint_asc(self, qfield):
-        '''Applies a hint for the query that it should use a 
+        ''' Applies a hint for the query that it should use a 
             (``qfield``, ASCENDING) index when performing the query.
+            
             **Parameters**:
                 * qfield: the instance of :class:`mongoalchemy.QueryField` to use as the key.
         '''
         return self.__hint(qfield, ASCENDING)
     
     def hint_desc(self, qfield):
-        '''Applies a hint for the query that it should use a 
+        ''' Applies a hint for the query that it should use a 
             (``qfield``, DESCENDING) index when performing the query.
+            
             **Parameters**:
                 * qfield: the instance of :class:`mongoalchemy.QueryField` to use as the key.
         '''
         return self.__hint(qfield, DESCENDING)
     
     def __hint(self, qfield, direction):
+        qfield = self.resolve_name(qfield)
         name = str(qfield)
         for n, _ in self.hints:
             if n == name:
@@ -148,27 +179,28 @@ class Query(object):
         return self
     
     def explain(self):
-        '''Executes an explain operation on the database for the current 
+        ''' Executes an explain operation on the database for the current 
             query and returns the raw explain object returned.
         '''
         return self.__get_query_result().cursor.explain()
     
     def all(self):
-        '''Return all of the results of a query in a list'''
-        return [obj for obj in self]
+        ''' Return all of the results of a query in a list'''
+        return [obj for obj in iter(self)]
     
     def distinct(self, key):
-        '''Execute this query and return all of the unique values of 
-            ``key``.
-        **Parameters**:
-            * key: the instance of :class:`mongoalchemy.QueryField` to use as the distinct key.
+        ''' Execute this query and return all of the unique values 
+            of ``key``.
+            
+            **Parameters**:
+                * key: the instance of :class:`mongoalchemy.QueryField` to use as the distinct key.
         '''
         return self.__get_query_result().cursor.distinct(str(key))
     
     def filter(self, *query_expressions):
         '''Apply the given query expressions to this query object
             
-            **Example**: ``s.query(SomeObj).filter(SomeObj.f.age > 10, SomeObj.f.blood_type == 'O')``
+            **Example**: ``s.query(SomeObj).filter(SomeObj.age > 10, SomeObj.blood_type == 'O')``
             
             **Parameters**:
                 * query_expressions: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
@@ -176,14 +208,17 @@ class Query(object):
             .. seealso:: :class:`~mongoalchemy.query_expression.QueryExpression` class
         '''
         for qe in query_expressions:
-            self._apply(qe)
+            if isinstance(qe, dict):
+                self._apply_dict(qe)
+            else:
+                self._apply(qe)
         return self
     
     def filter_by(self, **filters):
         ''' Filter for the names in ``filters`` being equal to the associated 
             values.  Cannot be used for sub-objects since keys must be strings'''
         for name, value in filters.iteritems():
-            self.filter(getattr(self.type, name) == value)
+            self.filter(self.resolve_name(name) == value)
         return self
     
     def count(self, with_limit_and_skip=False):
@@ -195,31 +230,38 @@ class Query(object):
         return self.__get_query_result().cursor.count(with_limit_and_skip=with_limit_and_skip)
     
     def fields(self, *fields):
-        '''Only return the specified fields from the object.  Accessing a \
+        ''' Only return the specified fields from the object.  Accessing a \
             field that was not specified in ``fields`` will result in a \
             :class:``mongoalchemy.document.FieldNotRetrieved`` exception being \
             raised
-        
-        **Parameters**:
-            * fields: Instances of :class:``mongoalchemy.query.QueryField`` specifying \
-                which fields to return
+            
+            **Parameters**:
+                * fields: Instances of :class:``mongoalchemy.query.QueryField`` specifying \
+                    which fields to return
         '''
         if self._fields == None:
             self._fields = set()
         for f in fields:
+            f = self.resolve_name(f)
             self._fields.add(f)
         self._fields.add(self.type.mongo_id)
         return self
     
     def _apply(self, qe):
+        ''' Apply a raw mongo query to the current raw query object'''
+        self._apply_dict(qe.obj)
+    
+    def _apply_dict(self, qe_dict):
         ''' Apply a query expression, updating the query object '''
-        for k, v in qe.obj.iteritems():
-            if k not in self.query:
-                self.query[k] = v
+        for k, v in qe_dict.iteritems():
+            k = self.resolve_name(k)
+            if not k in self.__query:
+                self.__query[k] = v
                 continue
-            if not isinstance(self.query[k], dict) or not isinstance(v, dict):
+            if not isinstance(self.__query[k], dict) or not isinstance(v, dict):
                 raise BadQueryException('Multiple assignments to a field must all be dicts.')
-            self.query[k].update(**v)
+            self.__query[k].update(**v)
+
     
     def ascending(self, qfield):
         ''' Sort the result based on ``qfield`` in ascending order.  These calls 
@@ -242,6 +284,7 @@ class Query(object):
         return self.__sort(qfield, DESCENDING)
     
     def __sort(self, qfield, direction):
+        qfield = self.resolve_name(qfield)
         name = str(qfield)
         for n, _ in self.sort:
             if n == name:
@@ -249,24 +292,24 @@ class Query(object):
         self.sort.append((name, direction))
         return self
     
-    # def not_(self, *query_expressions):
-    #     ''' Add a $not expression to the query, negating the query expressions 
-    #         given.  
-    #         
-    #         **Examples**: ``query.not_(SomeDocClass.f.age == 18)`` becomes ``{'$not' : { 'age' : 18 }}``
-    #         
-    #         **Parameters**:
-    #         * query_expressions: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
-    #         '''
-    #     for qe in query_expressions:
-    #         self.filter(qe.not_())
-    #     return self
+    def not_(self, *query_expressions):
+        ''' Add a $not expression to the query, negating the query expressions 
+            given.  
+            
+            **Examples**: ``query.not_(SomeDocClass.age <= 18)`` becomes ``{'age' : { '$not' : { '$gt' : 18 } }}``
+            
+            **Parameters**:
+              * query_expressions: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
+            '''
+        for qe in query_expressions:
+            self.filter(qe.not_())
+        return self
     
     def or_(self, first_qe, *qes):
         ''' Add a $not expression to the query, negating the query expressions 
             given.  The ``| operator`` on query expressions does the same thing
             
-            **Examples**: ``query.or_(SomeDocClass.f.age == 18, SomeDocClass.f.age == 17)`` becomes ``{'$or' : [{ 'age' : 18 }, { 'age' : 17 }]}``
+            **Examples**: ``query.or_(SomeDocClass.age == 18, SomeDocClass.age == 17)`` becomes ``{'$or' : [{ 'age' : 18 }, { 'age' : 17 }]}``
             
             **Parameters**:
                 * query_expressions: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
@@ -279,24 +322,28 @@ class Query(object):
     
     def in_(self, qfield, *values):
         ''' Check to see that the value of ``qfield`` is one of ``values``
+            
             **Parameters**:
                 * qfield: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
                 * values: Values should be python values which ``qfield`` \
                     understands
         '''
         # TODO: make sure that this field represents a list
-        self.filter(QueryExpression({ str(qfield) : { '$in' : [qfield.wrap(value) for value in values]}}))
+        qfield = self.resolve_name(qfield)
+        self.filter(QueryExpression({ qfield : { '$in' : [qfield.wrap_value(value) for value in values]}}))
         return self
 
     def nin(self, qfield, *values):
         ''' Check to see that the value of ``qfield`` is not one of ``values``
+            
             **Parameters**:
                 * qfield: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
                 * values: Values should be python values which ``qfield`` \
                     understands
         '''
         # TODO: make sure that this field represents a list
-        self.filter(QueryExpression({ str(qfield) : { '$nin' : [qfield.wrap(value) for value in values]}}))
+        qfield = self.resolve_name(qfield)
+        self.filter(QueryExpression({ qfield : { '$nin' : [qfield.wrap_value(value) for value in values]}}))
         return self
 
     
@@ -308,9 +355,9 @@ class Query(object):
         '''Refer to:  :func:`~mongoalchemy.update_expression.UpdateExpression.unset`'''
         return UpdateExpression(self).unset(qfield)
     
-    def inc(self, qfield, value=1):
+    def inc(self, *args, **kwargs):
         '''Refer to:  :func:`~mongoalchemy.update_expression.UpdateExpression.inc`'''
-        return UpdateExpression(self).inc(qfield, value=value)
+        return UpdateExpression(self).inc(*args, **kwargs)
     
     def append(self, qfield, value):
         '''Refer to:  :func:`~mongoalchemy.update_expression.UpdateExpression.append`'''
@@ -352,7 +399,8 @@ class QueryResult(object):
         return self.type.unwrap(value, fields=self.fields)
     
     def __getitem__(self, index):
-        return self.type.unwrap(self.cursor.__getitem__(index))
+        value = self.cursor.__getitem__(index)
+        return self.type.unwrap(value)
     
     def rewind(self):
         return self.cursor.rewind()
@@ -367,6 +415,7 @@ class QueryResult(object):
 class RemoveQuery(object):
     def __init__(self, type, session):
         ''' Execute a remove query to remove the matched objects from the database
+            
             **Parameters**:
                 * type: A subclass of class:`mongoalchemy.document.Document`
                 * db: The :class:`~mongoalchemy.session.Session` which this query is associated with.
@@ -375,9 +424,12 @@ class RemoveQuery(object):
         self.type = type
         self.safe = None
         self.get_last_args = {}
-        self.query = {}
         self.__query_obj = Query(type, session)
     
+    @property
+    def query(self):
+        return self.__query_obj.query
+    
     def set_safe(self, is_safe, **kwargs):
         ''' Set this remove to be safe.  It will call getLastError after the 
             remove to make sure it was successful.  ``**kwargs`` are parameters to 
@@ -396,14 +448,12 @@ class RemoveQuery(object):
         ''' Filter the remove expression with ``*query_expressions``, as in
             the ``Query`` filter method.'''
         self.__query_obj.filter(*query_expressions)
-        self.query = self.__query_obj.query
         return self
     
     def filter_by(self, **filters):
         ''' Filter for the names in ``filters`` being equal to the associated 
             values.  Cannot be used for sub-objects since keys must be strings'''
         self.__query_obj.filter_by(**filters)
-        self.query = self.__query_obj.query
         return self
     
     # def not_(self, *query_expressions):
@@ -415,7 +465,6 @@ class RemoveQuery(object):
         ''' Works the same as the query expression method ``or_``
         '''
         self.__query_obj.or_(first_qe, *qes)
-        self.query = self.__query_obj.query
         return self
     
     def in_(self, qfield, *values):
@@ -423,14 +472,12 @@ class RemoveQuery(object):
         '''
 
         self.__query_obj.in_(qfield, *values)
-        self.query = self.__query_obj.query
         return self
 
     def nin(self, qfield, *values):
         ''' Works the same as the query expression method ``nin_``
         '''
         self.__query_obj.nin(qfield, *values)
-        self.query = self.__query_obj.query
         return self
     
     
\ No newline at end of file
