@@ -27,25 +27,54 @@ class BadQueryException(Exception):
     '''
     pass
 
-# class QueryFieldSet(object):
-#     ''' Intermediate class used to allow access to create QueryField objects 
-#         from a subclass of Document.  Should generally be indirectly accessed 
-#         via ``Document.f``.
-#     '''
-#     def __init__(self, type, fields, parent=None):
-#         self.type = type
-#         self.fields = fields
-#         self.parent = parent
-#     
-#     def __getattr__(self, name):
-#         if name not in self.fields:
-#             raise BadQueryException('%s is not a field in %s' % (name, self.type.class_name()))
-#         return QueryField(name, self.fields[name], parent=self.parent)
+class FreeFormField(object):
+    has_subfields = True
+    no_real_attributes = True
+    def __init__(self, name=None):
+        self.__name = name
+        self.db_field = name
+    def __getattr__(self, name):
+        return FreeFormField(name=name)
+    def __getitem__(self, name):
+        return getattr(self, name)
+    @classmethod
+    def wrap_value(self, value):
+        return value
+    def subfields(self):
+        return FreeFormField(name=None)
+    def is_valid_wrap(*args): 
+        return True
+    is_valid_unwrap = is_valid_wrap
+    __contains__ = is_valid_wrap
+
+class FreeFormDoc(object):
+    def __init__(self, name):
+        self.__name = name
+    def __getattr__(self, name):
+        return QueryField(FreeFormField(name))
+    @classmethod
+    def unwrap(cls, value, *args, **kwargs):
+        return value
+    def get_collection_name(self):
+        return self.__name
+    def get_indexes(self):
+        return []        
+    mongo_id = FreeFormField(name='_id')
+
+Q = FreeFormDoc('')
 
 class QueryField(object):
     def __init__(self, type, parent=None):
         self.__type = type
         self.__parent = parent
+        self.__cached_id_value = None
+        self.__matched_index = False
+    
+    @property
+    def __cached_id(self):
+        if self.__cached_id_value == None:
+            self.__cached_id_value = str(self)
+        return self.__cached_id_value
     
     def _get_parent(self):
         return self.__parent
@@ -54,10 +83,16 @@ class QueryField(object):
         ''' Returns the underlying :class:`mongoalchemy.fields.Field` '''
         return self.__type
     
+    def matched_index(self):
+        ''' Represents the matched array index on a query with objects inside
+            of a list.  In the MongoDB docs, this is the ``$`` operator '''
+        self.__matched_index = True
+        return self
+    
     def __getattr__(self, name):
-        if hasattr(self.__type, name):
+        if not self.__type.no_real_attributes and hasattr(self.__type, name):
             return getattr(self.__type, name)
-
+        
         if not self.__type.has_subfields:
             raise AttributeError(name)
         
@@ -66,14 +101,13 @@ class QueryField(object):
             raise BadQueryException('%s is not a field in %s' % (name, self.__type.sub_type()))
         return QueryField(fields[name], parent=self)
     
-    # def wrap(self, value):
-    #     return self.__type.wrap(value)
-    
     def get_absolute_name(self):
         res = []
         current = self
         
         while type(current) != type(None):
+            if current.__matched_index:
+                res.append('$')
             res.append(current.get_type().db_field)
             current = current._get_parent()
         return '.'.join(reversed(res))
@@ -83,7 +117,7 @@ class QueryField(object):
             in ``values``.  Produces a MongoDB ``$in`` expression.
         '''
         return QueryExpression({
-            str(self) : { '$in' : [self.wrap(value) for value in values] }
+            self : { '$in' : [self.get_type().wrap_value(value) for value in values] }
         })
     
     def nin(self, *values):
@@ -91,29 +125,35 @@ class QueryField(object):
             in ``values``.  Produces a MongoDB ``$nin`` expression.
         '''
         return QueryExpression({
-            str(self) : { '$nin' : [self.wrap(value) for value in values] }
+            self : { '$nin' : [self.get_type().wrap_value(value) for value in values] }
         })
     
     def __str__(self):
         return self.get_absolute_name()
     
+    def __repr__(self):
+        return 'QueryField(%s)' % str(self)
+    
+    def __hash__(self):
+        return hash(self.__cached_id)
+    
     def __eq__(self, value):
         return self.eq_(value)
     def eq_(self, value):
         ''' Creates a query expression where ``this field == value`` 
         
-            .. note:: The prefered usage is via an operator: ``User.f.name == value``
+            .. note:: The prefered usage is via an operator: ``User.name == value``
         '''
-        if not self.get_type().is_valid_wrap(value):
-            raise BadQueryException('Invalid "value" for comparison against %s: %s' % (str(self), value))
-        return QueryExpression({ self.get_absolute_name() : self.wrap(value) })
+        if isinstance(value, QueryField):
+            return self.__cached_id == value.__cached_id
+        return QueryExpression({ self : self.get_type().wrap_value(value) })
     
     def __lt__(self, value):
         return self.lt_(value)
     def lt_(self, value):
         ''' Creates a query expression where ``this field < value`` 
         
-            .. note:: The prefered usage is via an operator: ``User.f.name < value``
+            .. note:: The prefered usage is via an operator: ``User.name < value``
         '''
         return self.__comparator('$lt', value)
     
@@ -122,7 +162,7 @@ class QueryField(object):
     def le_(self, value):
         ''' Creates a query expression where ``this field <= value`` 
         
-            .. note:: The prefered usage is via an operator: ``User.f.name <= value``
+            .. note:: The prefered usage is via an operator: ``User.name <= value``
         '''
         return self.__comparator('$lte', value)
     
@@ -131,8 +171,10 @@ class QueryField(object):
     def ne_(self, value):
         ''' Creates a query expression where ``this field != value`` 
         
-            .. note:: The prefered usage is via an operator: ``User.f.name != value``
+            .. note:: The prefered usage is via an operator: ``User.name != value``
         '''
+        if isinstance(value, QueryField):
+            return self.__cached_id != value.__cached_id
         return self.__comparator('$ne', value)
     
     def __gt__(self, value):
@@ -140,7 +182,7 @@ class QueryField(object):
     def gt_(self, value):
         ''' Creates a query expression where ``this field > value`` 
         
-            .. note:: The prefered usage is via an operator: ``User.f.name > value``
+            .. note:: The prefered usage is via an operator: ``User.name > value``
         '''
         return self.__comparator('$gt', value)
     
@@ -149,19 +191,16 @@ class QueryField(object):
     def ge_(self, value):
         ''' Creates a query expression where ``this field >= value`` 
         
-            .. note:: The prefered usage is via an operator: ``User.f.name >= value``
+            .. note:: The prefered usage is via an operator: ``User.name >= value``
         '''
         return self.__comparator('$gte', value)
     
     def __comparator(self, op, value):
-        try:
-            return QueryExpression({
-                self.get_absolute_name() : {
-                    op : self.wrap(value)
-                }
-            })
-        except BadValueException:
-            raise BadQueryException('Invalid "value" for %s comparison against %s: %s' % (self, op, value))
+        return QueryExpression({
+            self.get_absolute_name() : {
+                op : self.get_type().wrap(value)
+            }
+        })
 
     
 class QueryExpression(object):
@@ -173,22 +212,38 @@ class QueryExpression(object):
     '''
     def __init__(self, obj):
         self.obj = obj
-    # def not_(self):
-    #     '''Negates this instance's query expression using MongoDB's ``$not`` 
-    #         operator
-    #         
-    #         **Example**: ``(User.f.name == 'Jeff').not_()``
-    #         
-    #         .. note:: Another usage is via an operator, but parens are needed 
-    #             to get past precedence issues: ``~ (User.f.name == 'Jeff')``
-    #         '''
-    # 
-    #     return QueryExpression({
-    #             '$not' : self.obj
-    #         })
+    def not_(self):
+        '''Negates this instance's query expression using MongoDB's ``$not`` 
+            operator
+            
+            **Example**: ``(User.name == 'Jeff').not_()``
+            
+            .. note:: Another usage is via an operator, but parens are needed 
+                to get past precedence issues: ``~ (User.name == 'Jeff')``
+            '''
+        ret_obj = {}
+        for k, v in self.obj.iteritems():
+            if not isinstance(v, dict):
+                ret_obj[k] = {'$ne' : v }
+                continue
+            num_ops = len([x for x in v if x[0] == '$'])
+            if num_ops != len(v) and num_ops != 0:
+                raise BadQueryException('$ operator used in field name')
+            
+            if num_ops == 0:
+                ret_obj[k] = {'$ne' : v }
+                continue
+            
+            for op, value in v.iteritems():
+                k_dict = ret_obj.setdefault(k, {})
+                not_dict = k_dict.setdefault('$not', {})
+                not_dict[op] = value
+        
+            
+        return QueryExpression(ret_obj)
     
-    # def __invert__(self):
-    #     return self.not_()
+    def __invert__(self):
+        return self.not_()
     
     def __or__(self, expression):
         return self.or_(expression)
@@ -197,9 +252,9 @@ class QueryExpression(object):
         ''' Adds the given expression to this instance's MongoDB ``$or`` 
             expression, starting a new one if one does not exst
             
-            **Example**: ``(User.f.name == 'Jeff').or_(User.f.name == 'Jack')``
+            **Example**: ``(User.name == 'Jeff').or_(User.name == 'Jack')``
             
-            .. note:: The prefered usageis via an operator: ``User.f.name == 'Jeff' | User.f.name == 'Jack'``
+            .. note:: The prefered usageis via an operator: ``User.name == 'Jeff' | User.name == 'Jack'``
             
             '''
         
