@@ -63,8 +63,7 @@ class UpdateExpression(object):
         elif len(kwargs) != 0:
             ret = self
             for key, value in kwargs.iteritems():
-                qfield = getattr(self.query.type, key)
-                ret = ret._atomic_op('$set', qfield, value)
+                ret = ret._atomic_op('$set', key, value)
             return ret
         else:
             raise UpdateException('Invalid arguments for set.  Requires either two positional arguments or at least one keyword argument')
@@ -76,9 +75,22 @@ class UpdateExpression(object):
         # TODO: assert server version is >1.3.0
         return self._atomic_generic_op('$unset', qfield, True)
         
-    def inc(self, qfield, value=1):
+    def inc(self, *args, **kwargs):
         ''' Atomically increment ``qfield`` by ``value`` '''
-        return self._atomic_op('$inc', qfield, value)
+        pairs = []
+        if len(args) == 1:
+            pairs.append((args[0], 1))
+        elif len(args) == 2:
+            pairs.append(args)
+        elif len(kwargs) != 0:
+            pairs.extend([(k, v) for k, v in kwargs.iteritems()])
+        else:
+            raise UpdateException('Invalid arguments for set.  Requires either two positional arguments or at least one keyword argument')
+
+        ret = self
+        for qfield, value in pairs:
+            ret = self._atomic_op('$inc', qfield, value)
+        return ret
         
     def append(self, qfield, value):
         ''' Atomically append ``value`` to ``qfield``.  The operation will 
@@ -117,6 +129,7 @@ class UpdateExpression(object):
         return self._atomic_generic_op('$pop', qfield, -1)
     
     def _atomic_list_op_multivalue(self, op, qfield, *value):
+        qfield = self.query.resolve_name(qfield)
         if op not in qfield.valid_modifiers:
             raise InvalidModifierException(qfield, op)
         wrapped = []
@@ -128,6 +141,7 @@ class UpdateExpression(object):
         return self
     
     def _atomic_list_op(self, op, qfield, value):
+        qfield = self.query.resolve_name(qfield)
         if op not in qfield.valid_modifiers:
             raise InvalidModifierException(qfield, op)
         
@@ -137,6 +151,7 @@ class UpdateExpression(object):
         return self
     
     def _atomic_op(self, op, qfield, value):
+        qfield = self.query.resolve_name(qfield)
         if op not in qfield.valid_modifiers:
             raise InvalidModifierException(qfield, op)
 
@@ -146,6 +161,7 @@ class UpdateExpression(object):
         return self
     
     def _atomic_generic_op(self, op, qfield, value):
+        qfield = self.query.resolve_name(qfield)
         if op not in qfield.valid_modifiers:
             raise InvalidModifierException(qfield, op)
         
